/*******************************************************************************

  ioaccess.c: IO access code for Lanner Platfomr Digital IO program

  Lanner Platform Miscellaneous Utility
  Copyright(c) 2010- 2011 Lanner Electronics Inc.
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:
  1. Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer,
     without modification.
  2. Redistributions in binary form must reproduce at minimum a disclaimer
     similar to the "NO WARRANTY" disclaimer below ("Disclaimer") and any
     redistribution must be conditioned upon including a substantially
     similar Disclaimer requirement for further binary redistribution.
  3. Neither the names of the above-listed copyright holders nor the names
     of any contributors may be used to endorse or promote products derived
     from this software without specific prior written permission.

  Alternatively, this software may be distributed under the terms of the
  GNU General Public License ("GPL") version 2 as published by the Free
  Software Foundation.

  NO WARRANTY
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF NONINFRINGEMENT, MERCHANTIBILITY
  AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
  THE POSSIBILITY OF SUCH DAMAGES.

*******************************************************************************/



#include "../include/config.h"


#ifdef DJGPP

/* standard include file */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
/* For DOS DJGPP */
#include <dos.h>
#include <inlines/pc.h>

#else //DJGPP
/* For Linux */


#ifdef DIRECT_IO_ACCESS
/* For Linux direct io access code */
/* standard include file */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#if defined(LINUX_ENV)
#include <sys/io.h>
#endif

#if defined(FreeBSD_ENV)
#include <machine/cpufunc.h>
#endif


#include <time.h>
#include <stdint.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#define delay(x) usleep(x)
#endif


#ifdef MODULE

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <asm/io.h>
#include <linux/delay.h>

#undef delay
#define delay(x) mdelay(x)
#undef fprintf
#define fprintf(S, A)  printk(A)

#endif //MODULE


#ifdef KLD_MODULE

#include <sys/types.h>
#include <sys/param.h>
#include <sys/systm.h>
#include <sys/malloc.h>
#include <sys/kernel.h>
#include <sys/bus.h>
#include <sys/errno.h>

#include <machine/bus.h>
#include <machine/resource.h>

#endif

#endif

/* local include file */
#include "../include/ioaccess.h"

#if (defined(MODULE) || defined(DIRECT_IO_ACCESS) || defined(KLD_MODULE))

/*
 *------------------------------------------------------------------------------
 * LEB-7230 Version V0.1
 *
 * The IO interface for Digital DIO is connected to fintek SIO F81865.
 * Platform provide 2 digital input and 2 digital output. 
 * GPIO0,1 as input function, GPIO2,3 as output function
 * Refer to Fintek 81865 datasheet for details.
 * The truth table is defined as below:
 * DIO GPIO pins as follows:
 *        IN		  OUT
 * DIO		GP0		  GP2
 * DIO		GP1		  GP3
 *-----------------------------------------------------------------------------------
 */

/*
 * Device Depend Definition : F81865 SIO
*/
#define INDEX_PORT      0x2E
#define DATA_PORT       0x2F


#define GPIO0_BIT	(1 << 0)
#define GPIO1_BIT	(1 << 1)
#define GPIO2_BIT	(1 << 2)
#define GPIO3_BIT	(1 << 3)
#define GPIO_GPIO0_GPIO3_MASK (GPIO0_BIT | GPIO1_BIT | GPIO2_BIT | GPIO3_BIT)

//#define OUT_SHIFT 2
void enter_F81865_config(void)
{
        outportb(INDEX_PORT, 0x87); // Must Do It Twice
        outportb(INDEX_PORT, 0x87);
        return;
}

void exit_F81865_config(void)
{
        outportb(INDEX_PORT, 0xAA);
        return;
}

unsigned char read_F81865_reg(int LDN, int reg)
{
        unsigned char tmp = 0;

        enter_F81865_config();
        outportb(INDEX_PORT, 0x07); // LDN Register
        outportb(DATA_PORT, LDN); // Select LDNx
        outportb(INDEX_PORT, reg); // Select Register
        tmp = inportb( DATA_PORT); // Read Register
        exit_F81865_config();
        return tmp;
}


void write_F81865_reg(int LDN, int reg, int value)
{
        enter_F81865_config();
        outportb(INDEX_PORT, 0x07); // LDN Register
        outportb(DATA_PORT, LDN); // Select LDNx
        outportb(INDEX_PORT, reg); // Select Register
        outportb(DATA_PORT, value); // Write Register
        exit_F81865_config();
        return;
}

void dio_gpio_init(void)
{
        unsigned char tmp;
	/* Enable GPIO 0x/5x function */
	write_F81865_reg(0x06,0x26, 0x00);
	
	tmp=read_F81865_reg(0x06, 0x2A);
	tmp |= 0x08;
	write_F81865_reg(0x06, 0x2A, tmp);
	
	tmp=read_F81865_reg(0x06, 0x30);
	tmp |= 0x01;
	write_F81865_reg(0x06, 0x30, tmp);
	
	/* set GPIO0~1 as input function, GPIO2,3 as output */
	tmp=read_F81865_reg(0x06, 0xF0);
	tmp &= ~(GPIO0_BIT|GPIO1_BIT);
	tmp |= (GPIO2_BIT|GPIO3_BIT);
	write_F81865_reg(0x06, 0xF0, tmp);
        
	/* set GPIO0~3 as Open Drain */
	tmp=read_F81865_reg(0x06, 0xF3);
	tmp &= ~(GPIO_GPIO0_GPIO3_MASK);
	write_F81865_reg(0x06, 0xF3, tmp);
	
   /* set GPIO0~3 as Leve mode */     
  tmp=read_F81865_reg(0x06, 0xF4);
	tmp &= ~(GPIO_GPIO0_GPIO3_MASK);
	write_F81865_reg(0x06, 0xF4, tmp);
  	   
        return;
}
void dio_set_output(unsigned char out_value)
{
	unsigned char tmp;

	tmp = read_F81865_reg(0x06,0xF1);
	tmp &= ~(GPIO2_BIT|GPIO3_BIT);
	tmp |= (out_value << 2);
	write_F81865_reg(0x06, 0xF1, tmp);
        return;  
}

unsigned char dio_get_input(void)
{
        unsigned char tmp,tmp1;

        tmp=read_F81865_reg(0x06, 0xF2);
        return tmp;
}

#endif
